@@ -2,13 +2,15 @@ import typing
 from dataclasses import dataclass
 from typing import Any, Union
 
-from iwf.iwf_api.iwf_api.models import (
+from iwf.errors import WorkflowDefinitionError, NotRegisteredError
+from iwf.iwf_api.models import (
     ChannelRequestStatus,
     CommandResults as IdlCommandResults,
     TimerStatus,
 )
-from iwf.iwf_api.iwf_api.types import Unset
+from iwf.iwf_api.types import Unset
 from iwf.object_encoder import ObjectEncoder
+from iwf.type_store import TypeStore
 
 
 @dataclass
@@ -42,7 +44,7 @@ class CommandResults:
 
 def from_idl_command_results(
     idl_results: Union[Unset, IdlCommandResults],
-    internal_channel_types: dict[str, typing.Optional[type]],
+    internal_channel_types: TypeStore,
     signal_channel_types: dict[str, typing.Optional[type]],
     object_encoder: ObjectEncoder,
 ) -> CommandResults:
@@ -57,12 +59,20 @@ def from_idl_command_results(
 
     if not isinstance(idl_results.inter_state_channel_results, Unset):
         for inter in idl_results.inter_state_channel_results:
+
+            try:
+                val_type = internal_channel_types.get_type(inter.channel_name)
+            except NotRegisteredError as exception:
+                raise WorkflowDefinitionError(
+                    "internal channel is not registered: " + inter.channel_name
+                ) from exception
+
+            encoded = object_encoder.decode(inter.value, val_type)
+
             results.internal_channel_commands.append(
                 InternalChannelCommandResult(
                     inter.channel_name,
-                    object_encoder.decode(
-                        inter.value, internal_channel_types.get(inter.channel_name)
-                    ),
+                    encoded,
                     inter.request_status,
                     inter.command_id,
                 )
