@@ -1,6 +1,6 @@
 from dataclasses import dataclass, field
 from enum import Enum
-from typing import List, Optional
+from typing import List, Optional, Union
 
 
 class CommunicationMethodType(Enum):
@@ -13,17 +13,32 @@ class CommunicationMethod:
     name: str
     method_type: CommunicationMethodType
     value_type: Optional[type]
+    is_prefix: bool
 
     @classmethod
     def signal_channel_def(cls, name: str, value_type: type):
         return CommunicationMethod(
-            name, CommunicationMethodType.SignalChannel, value_type
+            name, CommunicationMethodType.SignalChannel, value_type, False
         )
 
     @classmethod
-    def internal_channel_def(cls, name: str, value_type: type):
+    def internal_channel_def(cls, name: str, value_type: Union[type, None]):
         return CommunicationMethod(
-            name, CommunicationMethodType.InternalChannel, value_type
+            name,
+            CommunicationMethodType.InternalChannel,
+            value_type if value_type is not None else type(None),
+            False,
+        )
+
+    @classmethod
+    def internal_channel_def_by_prefix(
+        cls, name_prefix: str, value_type: Union[type, None]
+    ):
+        return CommunicationMethod(
+            name_prefix,
+            CommunicationMethodType.InternalChannel,
+            value_type if value_type is not None else type(None),
+            True,
         )
 
 
