@@ -12,10 +12,13 @@ from iwf.errors import (
     parse_unexpected_error,
     process_http_error,
     process_workflow_abnormal_exit_error,
+    InvalidArgumentError,
 )
 from iwf.iwf_api import Client, errors
 from iwf.iwf_api.api.default import (
     post_api_v1_workflow_dataobjects_get,
+    post_api_v1_workflow_dataobjects_set,
+    post_api_v1_workflow_get,
     post_api_v1_workflow_reset,
     post_api_v1_workflow_rpc,
     post_api_v1_workflow_search,
@@ -59,9 +62,18 @@ from iwf.iwf_api.models import (
     WorkflowSetSearchAttributesRequest,
     WorkflowWaitForStateCompletionRequest,
 )
+from iwf.iwf_api.models.workflow_get_data_objects_response import (
+    WorkflowGetDataObjectsResponse,
+)
+from iwf.iwf_api.models.workflow_set_data_objects_request import (
+    WorkflowSetDataObjectsRequest,
+)
+from iwf.iwf_api.models.workflow_start_response import WorkflowStartResponse
 from iwf.iwf_api.types import Response
 from iwf.reset_workflow_type_and_options import ResetWorkflowTypeAndOptions
 from iwf.stop_workflow_options import StopWorkflowOptions
+from iwf.utils.iwf_typing import assert_not_unset
+from iwf.utils.persistence_utils import get_search_attribute_value
 
 
 @dataclass
@@ -76,6 +88,7 @@ class UnregisteredWorkflowOptions:
     initial_data_attributes: Optional[dict[str, Any]] = None
     wait_for_completion_state_execution_ids: Optional[list[str]] = None
     wait_for_completion_state_ids: Optional[list[str]] = None
+    initial_search_attributes: Optional[list[SearchAttribute]] = None
 
 
 T = TypeVar("T")
@@ -92,7 +105,7 @@ def add_http_status(name, value, phrase, description=""):
         HTTPStatus.__class__
     )  # store the enum's member parent class
     setattr(HTTPStatus, name, new_status)  # add it to the global HTTPStatus namespace
-    HTTPStatus._member_map_[name] = new_status  #  add it to the name=>member map
+    HTTPStatus._member_map_[name] = new_status  # add it to the name=>member map
     HTTPStatus._member_names_.append(
         name
     )  # append the names so it appears in __members__
@@ -157,6 +170,17 @@ class UnregisteredClient:
                     options.workflow_already_started_options
                 )
 
+            if options.initial_search_attributes:
+                for search_attribute in options.initial_search_attributes:
+                    val = get_search_attribute_value(
+                        assert_not_unset(search_attribute.value_type), search_attribute
+                    )
+                    if val is None:
+                        raise InvalidArgumentError(
+                            f"search attribute value is not set correctly for key {search_attribute.key} with value type {search_attribute.value_type}"
+                        )
+                start_options.search_attributes = options.initial_search_attributes
+
             if options.initial_data_attributes:
                 das = []
                 for key, value in options.initial_data_attributes.items():
@@ -177,11 +201,18 @@ class UnregisteredClient:
                     options.wait_for_completion_state_ids
                 )
 
-        response = post_api_v1_workflow_start.sync_detailed(
-            client=self.api_client,
-            json_body=request,
+        workflow_start_response = handler_error_and_return(
+            post_api_v1_workflow_start.sync_detailed(
+                client=self.api_client,
+                json_body=request,
+            )
         )
-        return handler_error_and_return(response)
+        if workflow_start_response is None or not isinstance(
+            workflow_start_response, WorkflowStartResponse
+        ):
+            raise RuntimeError("Failed to parse WorkflowStartResponse")
+
+        return workflow_start_response.workflow_run_id
 
     def get_simple_workflow_result_with_wait(
         self,
@@ -400,12 +431,27 @@ class UnregisteredClient:
         )
         return handler_error_and_return(response)
 
+    def get_workflow(
+        self,
+        workflow_id: str,
+        workflow_run_id: str,
+    ) -> WorkflowGetResponse:
+        request = WorkflowGetRequest(
+            workflow_id=workflow_id,
+            workflow_run_id=workflow_run_id,
+        )
+        response = post_api_v1_workflow_get.sync_detailed(
+            client=self.api_client,
+            json_body=request,
+        )
+        return handler_error_and_return(response)
+
     def get_workflow_data_attributes(
         self,
         workflow_id: str,
         workflow_run_id: str,
         attribute_keys: Optional[List[str]] = None,
-    ):
+    ) -> WorkflowGetDataObjectsResponse:
         request = WorkflowGetDataObjectsRequest(
             workflow_id=workflow_id,
             workflow_run_id=workflow_run_id,
@@ -418,6 +464,25 @@ class UnregisteredClient:
         )
         return handler_error_and_return(response)
 
+    def set_workflow_data_attributes(
+        self, workflow_id: str, workflow_run_id: str, attrs: dict[str, Any]
+    ):
+        objects = [
+            KeyValue(key, self.client_options.object_encoder.encode(value))
+            for key, value in attrs.items()
+        ]
+
+        request = WorkflowSetDataObjectsRequest(
+            workflow_id,
+            workflow_run_id,
+            objects,
+        )
+
+        response = post_api_v1_workflow_dataobjects_set.sync_detailed(
+            client=self.api_client, json_body=request
+        )
+        return handler_error_and_return(response)
+
     def search_workflow(
         self,
         request: WorkflowSearchRequest,
